@@ -1,25 +1,26 @@
 /* Test file for mpfr_add and mpfr_sub.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
-#define N 30000
+#define NUM 30000
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,13 +36,35 @@ static int usesp;
 static int
 test_add (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 {
+  int res;
+#ifdef CHECK_EXTERNAL
+  int ok = rnd_mode == GMP_RNDN && mpfr_number_p (b) && mpfr_number_p (c);
+  if (ok)
+    {
+      mpfr_print_raw (b);
+      printf (" ");
+      mpfr_print_raw (c);
+    }
+#endif
   if (usesp || MPFR_ARE_SINGULAR(b,c) || MPFR_SIGN(b) != MPFR_SIGN(c))
-    return mpfr_add (a, b, c, rnd_mode);
-  MPFR_CLEAR_FLAGS(a); /* clear flags */
-  if (MPFR_GET_EXP(b) < MPFR_GET_EXP(c))
-    return mpfr_add1(a, c, b, rnd_mode);
+    res = mpfr_add (a, b, c, rnd_mode);
   else
-    return mpfr_add1(a, b, c, rnd_mode);
+    {
+      MPFR_CLEAR_FLAGS(a); /* clear flags */
+      if (MPFR_GET_EXP(b) < MPFR_GET_EXP(c))
+        res = mpfr_add1(a, c, b, rnd_mode);
+      else
+        res = mpfr_add1(a, b, c, rnd_mode);
+    }
+#ifdef CHECK_EXTERNAL
+  if (ok)
+    {
+      printf (" ");
+      mpfr_print_raw (a);
+      printf ("\n");
+    }
+#endif
+  return res;
 }
 
 /* Parameter "z1" of check() used to be last in the argument list, but that
@@ -57,7 +80,7 @@ test_add (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 /* checks that x+y gives the right result with 53 bits of precision */
 static void
 pcheck (const char *xs, const char *ys, const char *zs, mp_rnd_t rnd_mode,
-	unsigned int px, unsigned int py, unsigned int pz)
+        unsigned int px, unsigned int py, unsigned int pz)
 {
   mpfr_t xx,yy,zz;
 
@@ -76,14 +99,14 @@ pcheck (const char *xs, const char *ys, const char *zs, mp_rnd_t rnd_mode,
               xs, ys, mpfr_print_rnd_mode (rnd_mode));
       exit (1);
     }
-  mpfr_clears (xx, yy, zz, NULL);
+  mpfr_clears (xx, yy, zz, (mpfr_ptr) 0);
 }
 
 static void
 check2b (const char *xs, int px,
-	 const char *ys, int py,
-	 const char *rs, int pz,
-	 mp_rnd_t rnd_mode)
+         const char *ys, int py,
+         const char *rs, int pz,
+         mp_rnd_t rnd_mode)
 {
   mpfr_t xx, yy, zz;
 
@@ -489,55 +512,55 @@ check_inexact (void)
       mpfr_set_prec (x, px);
       do
         {
-          mpfr_random (x);
+          mpfr_urandomb (x, RANDS);
         }
       while (mpfr_cmp_ui (x, 0) == 0);
       for (pu=2; pu<MAX_PREC; pu++)
-	{
-	  mpfr_set_prec (u, pu);
+        {
+          mpfr_set_prec (u, pu);
           do
             {
-              mpfr_random (u);
+              mpfr_urandomb (u, RANDS);
             }
           while (mpfr_cmp_ui (u, 0) == 0);
           {
               py = MPFR_PREC_MIN + (randlimb () % (MAX_PREC - 1));
-	      mpfr_set_prec (y, py);
-	      pz =  (mpfr_cmpabs (x, u) >= 0) ? MPFR_EXP(x) - MPFR_EXP(u)
-		: MPFR_EXP(u) - MPFR_EXP(x);
-	      /* x + u is exactly representable with precision
-		 abs(EXP(x)-EXP(u)) + max(prec(x), prec(u)) + 1 */
-	      pz = pz + MAX(MPFR_PREC(x), MPFR_PREC(u)) + 1;
-	      mpfr_set_prec (z, pz);
-	      rnd = (mp_rnd_t) RND_RAND();
-	      if (test_add (z, x, u, rnd))
-		{
-		  printf ("z <- x + u should be exact\n");
-		  printf ("x="); mpfr_print_binary (x); puts ("");
-		  printf ("u="); mpfr_print_binary (u); puts ("");
-		  printf ("z="); mpfr_print_binary (z); puts ("");
-		  exit (1);
-		}
-		{
-                  rnd = (mp_rnd_t) RND_RAND();
-		  inexact = test_add (y, x, u, rnd);
-		  cmp = mpfr_cmp (y, z);
-		  if (((inexact == 0) && (cmp != 0)) ||
-		      ((inexact > 0) && (cmp <= 0)) ||
-		      ((inexact < 0) && (cmp >= 0)))
-		    {
-		      printf ("Wrong inexact flag for rnd=%s\n",
+              mpfr_set_prec (y, py);
+              pz =  (mpfr_cmpabs (x, u) >= 0) ? MPFR_EXP(x) - MPFR_EXP(u)
+                : MPFR_EXP(u) - MPFR_EXP(x);
+              /* x + u is exactly representable with precision
+                 abs(EXP(x)-EXP(u)) + max(prec(x), prec(u)) + 1 */
+              pz = pz + MAX(MPFR_PREC(x), MPFR_PREC(u)) + 1;
+              mpfr_set_prec (z, pz);
+              rnd = RND_RAND ();
+              if (test_add (z, x, u, rnd))
+                {
+                  printf ("z <- x + u should be exact\n");
+                  printf ("x="); mpfr_print_binary (x); puts ("");
+                  printf ("u="); mpfr_print_binary (u); puts ("");
+                  printf ("z="); mpfr_print_binary (z); puts ("");
+                  exit (1);
+                }
+                {
+                  rnd = RND_RAND ();
+                  inexact = test_add (y, x, u, rnd);
+                  cmp = mpfr_cmp (y, z);
+                  if (((inexact == 0) && (cmp != 0)) ||
+                      ((inexact > 0) && (cmp <= 0)) ||
+                      ((inexact < 0) && (cmp >= 0)))
+                    {
+                      printf ("Wrong inexact flag for rnd=%s\n",
                               mpfr_print_rnd_mode(rnd));
-		      printf ("expected %d, got %d\n", cmp, inexact);
-		      printf ("x="); mpfr_print_binary (x); puts ("");
-		      printf ("u="); mpfr_print_binary (u); puts ("");
-		      printf ("y=  "); mpfr_print_binary (y); puts ("");
-		      printf ("x+u="); mpfr_print_binary (z); puts ("");
-		      exit (1);
-		    }
-		}
-	    }
-	}
+                      printf ("expected %d, got %d\n", cmp, inexact);
+                      printf ("x="); mpfr_print_binary (x); puts ("");
+                      printf ("u="); mpfr_print_binary (u); puts ("");
+                      printf ("y=  "); mpfr_print_binary (y); puts ("");
+                      printf ("x+u="); mpfr_print_binary (z); puts ("");
+                      exit (1);
+                    }
+                }
+            }
+        }
     }
 
   mpfr_clear (x);
@@ -660,7 +683,7 @@ check_1111 (void)
 
   mpfr_init2 (one, MPFR_PREC_MIN);
   mpfr_set_ui (one, 1, GMP_RNDN);
-  for (n = 0; n < N; n++)
+  for (n = 0; n < NUM; n++)
     {
       mp_prec_t prec_a, prec_b, prec_c;
       mp_exp_t tb=0, tc, diff;
@@ -683,7 +706,7 @@ check_1111 (void)
           mpfr_div_2ui (b, one, tb, GMP_RNDN);
           if (sb == 2)
             mpfr_neg (b, b, GMP_RNDN);
-          mpfr_add (b, b, one, GMP_RNDN);
+          test_add (b, b, one, GMP_RNDN);
         }
       else
         mpfr_set (b, one, GMP_RNDN);
@@ -692,13 +715,13 @@ check_1111 (void)
       sc = randlimb () % 2;
       if (sc)
         mpfr_neg (c, c, GMP_RNDN);
-      mpfr_add (c, c, one, GMP_RNDN);
+      test_add (c, c, one, GMP_RNDN);
       diff = (randlimb () % (2*m)) - m;
       mpfr_mul_2si (c, c, diff, GMP_RNDN);
-      rnd_mode = (mp_rnd_t) RND_RAND ();
+      rnd_mode = RND_RAND ();
       inex_a = test_add (a, b, c, rnd_mode);
       mpfr_init2 (s, MPFR_PREC_MIN + 2*m);
-      inex_s = mpfr_add (s, b, c, GMP_RNDN); /* exact */
+      inex_s = test_add (s, b, c, GMP_RNDN); /* exact */
       if (inex_s)
         {
           printf ("check_1111: result should have been exact.\n");
@@ -769,7 +792,7 @@ check_1minuseps (void)
               mpfr_div_ui (c, c, prec_a[ia] + supp_b[ic], GMP_RNDN);
               inex_a = test_add (a, b, c, (mp_rnd_t) rnd_mode);
               mpfr_init2 (s, 256);
-              inex_s = mpfr_add (s, b, c, GMP_RNDN); /* exact */
+              inex_s = test_add (s, b, c, GMP_RNDN); /* exact */
               if (inex_s)
                 {
                   printf ("check_1minuseps: result should have been exact "
@@ -807,62 +830,62 @@ tests (void)
   check64();
 
   check("293607738.0", "1.9967571564050541e-5", GMP_RNDU, 64, 53, 53,
-	"2.9360773800002003e8");
+        "2.9360773800002003e8");
   check("880524.0", "-2.0769715792901673e-5", GMP_RNDN, 64, 53, 53,
-	"8.8052399997923023e5");
+        "8.8052399997923023e5");
   check("1196426492.0", "-1.4218093058435347e-3", GMP_RNDN, 64, 53, 53,
-	"1.1964264919985781e9");
+        "1.1964264919985781e9");
   check("982013018.0", "-8.941829477291838e-7", GMP_RNDN, 64, 53, 53,
-	"9.8201301799999905e8");
+        "9.8201301799999905e8");
   check("1092583421.0", "1.0880649218158844e9", GMP_RNDN, 64, 53, 53,
-	"2.1806483428158846e9");
+        "2.1806483428158846e9");
   check("1.8476886419022969e-6", "961494401.0", GMP_RNDN, 53, 64, 53,
-	"9.6149440100000179e8");
+        "9.6149440100000179e8");
   check("-2.3222118418069868e5", "1229318102.0", GMP_RNDN, 53, 64, 53,
-	"1.2290858808158193e9");
+        "1.2290858808158193e9");
   check("-3.0399171300395734e-6", "874924868.0", GMP_RNDN, 53, 64, 53,
-	"8.749248679999969e8");
+        "8.749248679999969e8");
   check("9.064246624706179e1", "663787413.0", GMP_RNDN, 53, 64, 53,
-	"6.6378750364246619e8");
+        "6.6378750364246619e8");
   check("-1.0954322421551264e2", "281806592.0", GMP_RNDD, 53, 64, 53,
-	"2.8180648245677572e8");
+        "2.8180648245677572e8");
   check("5.9836930386056659e-8", "1016217213.0", GMP_RNDN, 53, 64, 53,
-	"1.0162172130000001e9");
+        "1.0162172130000001e9");
   check("-1.2772161928500301e-7", "1237734238.0", GMP_RNDN, 53, 64, 53,
-	"1.2377342379999998e9");
+        "1.2377342379999998e9");
   check("-4.567291988483277e8", "1262857194.0", GMP_RNDN, 53, 64, 53,
-	"8.0612799515167236e8");
+        "8.0612799515167236e8");
   check("4.7719471752925262e7", "196089880.0", GMP_RNDN, 53, 53, 53,
-	"2.4380935175292528e8");
+        "2.4380935175292528e8");
   check("4.7719471752925262e7", "196089880.0", GMP_RNDN, 53, 64, 53,
-	"2.4380935175292528e8");
+        "2.4380935175292528e8");
   check("-1.716113812768534e-140", "1271212614.0", GMP_RNDZ, 53, 64, 53,
-	"1.2712126139999998e9");
+        "1.2712126139999998e9");
   check("-1.2927455200185474e-50", "1675676122.0", GMP_RNDD, 53, 64, 53,
-	"1.6756761219999998e9");
+        "1.6756761219999998e9");
 
   check53("1.22191250737771397120e+20", "948002822.0", GMP_RNDN,
-	  "122191250738719408128.0");
+          "122191250738719408128.0");
   check53("9966027674114492.0", "1780341389094537.0", GMP_RNDN,
-	  "11746369063209028.0");
+          "11746369063209028.0");
   check53("2.99280481918991653800e+272", "5.34637717585790933424e+271",
-	  GMP_RNDN, "3.5274425367757071711e272");
+          GMP_RNDN, "3.5274425367757071711e272");
   check_same();
   check53("6.14384195492641560499e-02", "-6.14384195401037683237e-02",
-	  GMP_RNDU, "9.1603877261370314499e-12");
+          GMP_RNDU, "9.1603877261370314499e-12");
   check53("1.16809465359248765399e+196", "7.92883212101990665259e+196",
-	  GMP_RNDU, "9.0969267746123943065e196");
+          GMP_RNDU, "9.0969267746123943065e196");
   check53("3.14553393112021279444e-67", "3.14553401015952024126e-67", GMP_RNDU,
-	  "6.2910679412797336946e-67");
+          "6.2910679412797336946e-67");
 
   check53("5.43885304644369509058e+185","-1.87427265794105342763e-57",GMP_RNDN,
-	  "5.4388530464436950905e185");
+          "5.4388530464436950905e185");
   check53("5.43885304644369509058e+185","-1.87427265794105342763e-57",GMP_RNDZ,
-	  "5.4388530464436944867e185");
+          "5.4388530464436944867e185");
   check53("5.43885304644369509058e+185","-1.87427265794105342763e-57",GMP_RNDU,
-	  "5.4388530464436950905e185");
+          "5.4388530464436950905e185");
   check53("5.43885304644369509058e+185","-1.87427265794105342763e-57",GMP_RNDD,
-	  "5.4388530464436944867e185");
+          "5.4388530464436944867e185");
 
   check2b("1.001010101110011000000010100101110010111001010000001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000e358",187,
           "-1.11100111001101100010001111111110101101110001000000000000000000000000000000000000000000e160",87,
@@ -987,50 +1010,50 @@ tests (void)
 
   /* Checking double precision (53 bits) */
   check53("-8.22183238641455905806e-19", "7.42227178769761587878e-19",GMP_RNDD,
-	  "-7.9956059871694317927e-20");
+          "-7.9956059871694317927e-20");
   check53("5.82106394662028628236e+234","-5.21514064202368477230e+89",GMP_RNDD,
-	  "5.8210639466202855763e234");
+          "5.8210639466202855763e234");
   check53("5.72931679569871602371e+122","-5.72886070363264321230e+122",
-	  GMP_RNDN, "4.5609206607281141508e118");
+          GMP_RNDN, "4.5609206607281141508e118");
   check53("-5.09937369394650450820e+238", "2.70203299854862982387e+250",
-	  GMP_RNDD, "2.7020329985435301323e250");
+          GMP_RNDD, "2.7020329985435301323e250");
   check53("-2.96695924472363684394e+27", "1.22842938251111500000e+16",GMP_RNDD,
-	  "-2.96695924471135255027e27");
+          "-2.96695924471135255027e27");
   check53("1.74693641655743793422e-227", "-7.71776956366861843469e-229",
-	  GMP_RNDN, "1.669758720920751867e-227");
+          GMP_RNDN, "1.669758720920751867e-227");
   /*  x = -7883040437021647.0; for (i=0; i<468; i++) x = x / 2.0;*/
   check53("-1.03432206392780011159e-125", "1.30127034799251347548e-133",
-	  GMP_RNDN,
-	  "-1.0343220509150965661100887242027378881805094180354e-125");
+          GMP_RNDN,
+          "-1.0343220509150965661100887242027378881805094180354e-125");
   check53("1.05824655795525779205e+71", "-1.06022698059744327881e+71",GMP_RNDZ,
-	  "-1.9804226421854867632e68");
+          "-1.9804226421854867632e68");
   check53("-5.84204911040921732219e+240", "7.26658169050749590763e+240",
-	  GMP_RNDD, "1.4245325800982785854e240");
+          GMP_RNDD, "1.4245325800982785854e240");
   check53("1.00944884131046636376e+221","2.33809162651471520268e+215",GMP_RNDN,
-	  "1.0094511794020929787e221");
+          "1.0094511794020929787e221");
   /*x = 7045852550057985.0; for (i=0; i<986; i++) x = x / 2.0;*/
   check53("4.29232078932667367325e-278",
-	  "1.0773525047389793833221116707010783793203080117586e-281"
-	  , GMP_RNDU, "4.2933981418314132787e-278");
+          "1.0773525047389793833221116707010783793203080117586e-281"
+          , GMP_RNDU, "4.2933981418314132787e-278");
   check53("5.27584773801377058681e-80", "8.91207657803547196421e-91", GMP_RNDN,
-	  "5.2758477381028917269e-80");
+          "5.2758477381028917269e-80");
   check53("2.99280481918991653800e+272", "5.34637717585790933424e+271",
-	  GMP_RNDN, "3.5274425367757071711e272");
+          GMP_RNDN, "3.5274425367757071711e272");
   check53("4.67302514390488041733e-184", "2.18321376145645689945e-190",
-	  GMP_RNDN, "4.6730273271186420541e-184");
+          GMP_RNDN, "4.6730273271186420541e-184");
   check53("5.57294120336300389254e+71", "2.60596167942024924040e+65", GMP_RNDZ,
-	  "5.5729438093246831053e71");
+          "5.5729438093246831053e71");
   check53("6.6052588496951015469e24", "4938448004894539.0", GMP_RNDU,
-	  "6.6052588546335505068e24");
+          "6.6052588546335505068e24");
   check53("1.23056185051606761523e-190", "1.64589756643433857138e-181",
-	  GMP_RNDU, "1.6458975676649006598e-181");
+          GMP_RNDU, "1.6458975676649006598e-181");
   check53("2.93231171510175981584e-280", "3.26266919161341483877e-273",
-	  GMP_RNDU, "3.2626694848445867288e-273");
+          GMP_RNDU, "3.2626694848445867288e-273");
   check53("5.76707395945001907217e-58", "4.74752971449827687074e-51", GMP_RNDD,
-	  "4.747530291205672325e-51");
+          "4.747530291205672325e-51");
   check53("277363943109.0", "11.0", GMP_RNDN, "277363943120.0");
   check53("1.44791789689198883921e-140", "-1.90982880222349071284e-121",
-	  GMP_RNDN, "-1.90982880222349071e-121");
+          GMP_RNDN, "-1.90982880222349071e-121");
 
 
   /* tests for particular cases (Vincent Lefevre, 22 Aug 2001) */
@@ -1045,17 +1068,24 @@ tests (void)
   check_1minuseps ();
 }
 
+#define TEST_FUNCTION test_add
+#define TWO_ARGS
+#define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), randlimb () % 100)
+#include "tgeneric.c"
+
 int
 main (int argc, char *argv[])
 {
   tests_start_mpfr ();
-  mpfr_test_init ();
 
   usesp = 0;
   tests ();
 
+#ifndef CHECK_EXTERNAL /* no need to check twice */
   usesp = 1;
   tests ();
+#endif
+  test_generic (2, 1000, 100);
 
   tests_end_mpfr ();
   return 0;
