@@ -1,22 +1,22 @@
 /* Test file for mpfr_set.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -27,7 +27,7 @@ MA 02110-1301, USA. */
 
 /* Maybe better create its own test file ? */
 static void
-check_neg_special ()
+check_neg_special (void)
 {
   mpfr_t x;
   mpfr_init (x);
@@ -42,6 +42,9 @@ check_neg_special ()
   mpfr_clear (x);
 }
 
+#define TEST_FUNCTION mpfr_set
+#include "tgeneric.c"
+
 int
 main (void)
 {
@@ -112,7 +115,7 @@ main (void)
   for (p=2; p<500; p++)
     {
       mpfr_set_prec (x, p);
-      mpfr_random (x);
+      mpfr_urandomb (x, RANDS);
       if (randlimb () % 2)
         mpfr_neg (x, x, GMP_RNDN);
       for (q=2; q<2*p; q++)
@@ -141,6 +144,8 @@ main (void)
 
   check_neg_special ();
 
+  test_generic (2, 1000, 10);
+
   tests_end_mpfr ();
   return 0;
 }
