@@ -1,22 +1,22 @@
 /* Test file for mpfr_zeta_ui.
 
-Copyright 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -47,7 +47,8 @@ main (int argc, char *argv[])
   if (argc >= 3) /* tzeta_ui n prec [rnd] */
     {
       mpfr_set_prec (x, atoi (argv[2]));
-      mpfr_zeta_ui (x, atoi (argv[1]), (argc > 3) ? atoi (argv[3]) : GMP_RNDN);
+      mpfr_zeta_ui (x, atoi (argv[1]),
+                    argc > 3 ? (mp_rnd_t) atoi (argv[3]) : GMP_RNDN);
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN);
       printf ("\n");
       goto clear_and_exit;
